--Simple spotkiller. Will kill the spot if xin and yin haven't been both
--positive and negative for 1/3th second.

--Because in this design, the spotkiller won't kill just the spot but will cut
--all power to both the CRT and the deflection amps, it will make 'poweringup'
--high for a few secs when the crtenable goes back high. The idea is that this signal
--is used to center the beam and disable the spot so the hardware can power up in peace.


-- Black Widow arcade hardware implemented in an FPGA
-- (C) 2012 Jeroen Domburg (jeroen AT spritesmods.com)
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity spotkiller is
    Port ( poweringup : out STD_LOGIC;
			  reset: in STD_LOGIC;
			  clk_12 : in  STD_LOGIC;
           xin : in  STD_LOGIC_VECTOR(9 downto 0);
           yin : in  STD_LOGIC_VECTOR(9 downto 0);
           crtenable : out  STD_LOGIC);
end spotkiller;

architecture Behavioral of spotkiller is
	signal cnt: STD_LOGIC_VECTOR(19 downto 0);
	signal powerupcnt: STD_LOGIC_VECTOR(3 downto 0);
	signal xpos: STD_LOGIC;
	signal xneg: STD_LOGIC;
	signal ypos: STD_LOGIC;
	signal yneg: STD_LOGIC;
begin
	process(clk_12)
	begin
		if reset='1' then
			powerupcnt<="0000";
			xpos<='0';
			xneg<='0';
			ypos<='0';
			yneg<='0';
			cnt<=(others => '0');
		elsif rising_edge(clk_12) then
			cnt<=cnt+"1";
			if cnt="00000000000000000000" then
				if xpos='1' and ypos='1' and xneg='1' and yneg='1' then
					crtenable<='1';
					if powerupcnt/="1111" then
						powerupcnt<=powerupcnt+"0001";
						poweringup<='1';
					else
						poweringup<='0';
					end if;
				else
					crtenable<='0';
					poweringup<='0';
					powerupcnt<="0000";
				end if;
				xpos<='0';
				xneg<='0';
				ypos<='0';
				yneg<='0';
			end if;

			if xin(9)='1' then
				xpos<='1';
			else
				xneg<='1';
			end if;
			if yin(9)='1' then
				ypos<='1';
			else
				yneg<='1';
			end if;

		end if;
	end process;
end Behavioral;

